
local AlertBase4U = require("AlertBase4U")

local AlertOK4U = class("AlertOK4U", AlertBase4U)

function AlertOK4U:ctor(options)
    AlertOK4U.super.ctor(self)

    self._alertNode:opacity(255)
    self._alertNode:scale(1)

    local descText = options.desc or ""
    local okText = options.ok or z.ResourceManager:getInstance():getLocalizedString("ok")
    local okCallback = options.okCallback or nil

    local bg = cc.ui.UIPushButton.new("image/white_unit.webp")
        :setButtonSize(display.width, display.height)
        :align(display.CENTER, 0, 0)
        :addTo(self:getBGNode())
    bg:setColor(cc.c3b(0, 0, 0))
    bg:setOpacity(200)

    -- 框
    self._alertBG = display.newSprite("image/common_alert_bg.png")
        :align(display.CENTER, 0, 0)
        :addTo(self:getAlertNode())
    self._alertBG:setScaleY(0)

    self._alertContent = display.newNode()
        :hide()
        :addTo(self:getAlertNode())

    -- desc
    local descLabel = cc.ui.UILabel.new({
            text = descText, 
            size = 36,
            align = cc.ui.TEXT_ALIGN_CENTER})
        :align(display.CENTER, 0, 0)
        :addTo(self._alertContent)
    descLabel:setDimensions(900, 0)

    -- ok button
    local buttonImages = {
        normal = "image/common_alert_button_1.png",
        pressed = "image/common_alert_button_2.png",
    }
    local button = cc.ui.UIPushButton.new(buttonImages, {scale9 = true})
        :onButtonClicked(function() 
            if okCallback then okCallback() end
            self:actionExit()
        end)
        :pos(0, -100)
        :addTo(self._alertContent)
    local okLabel = cc.ui.UILabel.new({text = okText, size = 32})
        :addTo(self._alertContent)
    local okSprite = display.newSprite("image/common_alert_ok_icon.png")
        :addTo(self._alertContent)
    local width = okLabel:getContentSize().width + okSprite:getContentSize().width + 10
    okLabel:align(display.RIGHT_CENTER, 0 + width / 2, -100)
    okSprite:align(display.LEFT_CENTER, 0 - width / 2, -100)
    button:setButtonSize(math.max(width + 50, 180), 66)
end


function AlertOK4U:actionEnter()
    self._exiting = false

    self:show()

    self._bgNode:stopAllActions()
    self._bgNode:runAction(
        cca.fadeTo(0.2, 1))

    self._alertBG:stopAllActions()
    self._alertBG:runAction(cca.scaleTo(0.2, 1, 1))

    self._alertContent:stopAllActions()
    self._alertContent:runAction(cca.seq({
        cca.delay(0.2),
        cca.show()
        }))
end

function AlertOK4U:actionExit(needRemove)
    local needRemove = needRemove
    if needRemove == nil then needRemove = true end

    if self._exiting then return end
    self._exiting = true

    self._alertBG:stopAllActions()
    self._alertBG:runAction(cca.scaleTo(0.2, 1, 0))

    self._alertContent:stopAllActions()
    self._alertContent:runAction(cca.hide())

    self._bgNode:stopAllActions()
    self._bgNode:runAction(cca.fadeTo(0.2, 0))
    
    if needRemove then
        self:runAction(cca.seq({
            cca.delay(0.2),
            cca.hide(),
            cca.removeSelf()
            }))
    else
        self:runAction(cca.seq({
            cca.delay(0.2),
            cca.hide(),
            }))
    end
end

return AlertOK4U