------------------------------------------------------------------------------
--Load origin framework
------------------------------------------------------------------------------
-- cc.LuaLoadChunksFromZIP("res/framework_precompiled.zip")

------------------------------------------------------------------------------
--If you would update the modoules which have been require here,
--you can reset them, and require them again in modoule "AppMain"
------------------------------------------------------------------------------

function __G__TRACKBACK__(errorMessage)
    print("----------------------------------------")
    print("LUA ERROR: " .. tostring(errorMessage) .. "\n")
    print(debug.traceback("", 2))
    print("----------------------------------------")
end

package.path = package.path .. ";src/"
package.path = package.path .. ";src/?.lua"
package.path = package.path .. ';src/app/protobuf/?.lua'
package.path = package.path .. ';src/app/protocol/?.lua'

cc.FileUtils:getInstance():setPopupNotify(false)
cc.FileUtils:getInstance():addSearchPath("src/", true)
cc.FileUtils:getInstance():addSearchPath("", true)

-- print("------------------------------------------")
-- for k, v in pairs(package.loaded) do
--     print(k)
-- end
-- print("------------------------------------------")

require("config")
require("cocos.init")
require("framework.init")

-- for k, v in pairs(package.loaded) do
--     print(k)
-- end
-- print("------------------------------------------")

-- dump(cc.FileUtils:getInstance():getSearchPaths())

cc.FileUtils:getInstance():addSearchPath(device.writablePath .. "update/res/")
cc.FileUtils:getInstance():addSearchPath("res/")

------------------------------------------------------------------------------
--define UpdateScene
------------------------------------------------------------------------------
local UpdateScene = class("UpdateScene", function()
    return display.newScene("UpdateScene")
end)

UpdateScene.NEED_UPDATE = false
UpdateScene.SERVER_URL = "http://192.168.1.101:8080/"
-- local param = "?dev="..device.platform
UpdateScene.PARAM = ""
UpdateScene.FLIST_FILENAME = "flist"
UpdateScene.UPDATE_TEMP_POSTFIX = ".update"
UpdateScene.UPDATE_PATH = "update/"
UpdateScene.UPDATE_RES_PATH = "update_res/"
UpdateScene.DOWNLOADED_LIST = {}

local function hex(s)
    s = string.gsub(s,"(.)",function (x) return string.format("%02X",string.byte(x)) end)
    return s
end

local function isFileExists(path)
    local file = io.open(path, "rb")
    if file then
        io.close(file)
        return true
    else
        return false
    end
end

local function makeDir(path)
    printf("------------makeDir: %s", path)
    local tempPath = path
    local head = ""
    repeat
        -- printf("------------tempPath: %s, head: %s", tempPath, head)
        local firstDelimiter = string.find(tempPath, "/")

        if not firstDelimiter then
            if not isFileExists(head .. tempPath) then
                lfs.mkdir(head .. tempPath)
            end
            return
        else
            local tempDir = string.sub(tempPath, 1, firstDelimiter - 1)
            head = head .. tempDir .. "/"
            if not isFileExists(head) then
                lfs.mkdir(head)
            end
            tempPath = string.sub(tempPath, firstDelimiter + 1, #tempPath)
        end
    until not tempPath
end

local function readFile(path)
    local file = io.open(path, "rb")
    if file then
        local content = file:read("*all")
        io.close(file)
        return content
    end
    return nil
end

local function removeFile(path)
    printf("------------removeFile: %s", path)
    os.remove(path)
    -- io.writefile(path, "")
    -- if device.platform == "windows" then
    --     --os.execute("del " .. string.gsub(path, '/', '\\'))
    -- else
    --     os.execute("rm " .. path)
    -- end
end

local function removeDir(path)
    print("------------removeDir: ", path)
    if isFileExists(path) then
        local function _rmdir(path)
            local iter, dir_obj = lfs.dir(path)
            while true do
                local dir = iter(dir_obj)
                if dir == nil then break end
                if dir ~= "." and dir ~= ".." then
                    local curDir = path..dir
                    local mode = lfs.attributes(curDir, "mode") 
                    if mode == "directory" then
                        _rmdir(curDir.."/")
                    elseif mode == "file" then
                        os.remove(curDir)
                    end
                end
            end
            local succ, des = os.remove(path)
            -- if des then print(des) end
            return succ
        end
        _rmdir(path)
    end
    return true
end

local function checkFile(fileName, cryptoCode)
    -- print("------------checkFile:", fileName)
    -- print("------------cryptoCode:", cryptoCode)

    if not io.exists(fileName) then
        return false
    end

    local data = readFile(fileName)
    if data == nil then
        return false
    end

    if cryptoCode == nil then
        return true
    end

    local ms = crypto.md5(hex(data))
    -- print("------------file cryptoCode:", ms)
    if ms == cryptoCode then
        return true
    end

    return false
end

local function checkDirOK(path)
    require "lfs"
    local oldpath = lfs.currentdir()
    -- print("------------old path------> "..oldpath)

    if lfs.chdir(path) then
        lfs.chdir(oldpath)
        -- print("------------path check OK------> "..path)
        return true
    end

    if lfs.mkdir(path) then
        -- print("------------path create OK------> "..path)
        return true
    end
end

function UpdateScene:ctor()
    self.path = device.writablePath .. UpdateScene.UPDATE_PATH
        --if device.platform == "android" then
        --    self.path = "/mnt/sdcard/quick_x_update/"
        --end

    local label = cc.ui.UILabel.new({
        UILabelType = 2,
        text = "Loading...",
        size = 64,
        align = cc.ui.TEXT_ALIGN_CENTER
    })
    :align(display.CENTER, display.cx, display.cy)
    self:addChild(label)
end

function UpdateScene:updateFiles()
    -- print("------------updateFiles")
    local data = readFile(self.newListFile)
    removeFile(self.curListFile)
    io.writefile(self.curListFile, data)
    self.fileList = dofile(self.curListFile)
    if self.fileList == nil then
        self:endProcess()
        return
    end
    removeFile(self.newListFile)

    for i, v in ipairs(UpdateScene.DOWNLOADED_LIST) do
        -- print(i,v)
        local data = readFile(v)
        local fn = string.sub(v, 1, - string.len(UpdateScene.UPDATE_TEMP_POSTFIX) - 1)
        -- printf("------------fn: %s", fn)
        removeFile(fn)
        io.writefile(fn, data)
        removeFile(v)
    end
    self:endProcess()
end

function UpdateScene:reqNextFile()
    -- print("------------reqNextFile")
    self.numFileCheck = self.numFileCheck + 1
    self.curStageFile = self.fileListNew.stage[self.numFileCheck]
    if self.curStageFile and self.curStageFile.name then
        local fn = self.path .. self.curStageFile.name
        if checkFile(fn, self.curStageFile.code) then
            self:reqNextFile()
            return
        end
        fn = fn .. UpdateScene.UPDATE_TEMP_POSTFIX
        -- printf("------------fn: %s", fn)
        if checkFile(fn, self.curStageFile.code) then
            table.insert(UpdateScene.DOWNLOADED_LIST, fn)
            self:reqNextFile()
            return
        end
        self:requestFromServer(self.fileListNew.ver .. "/" .. self.curStageFile.name)
        return
    end

    self:updateFiles()
end

function UpdateScene:onEnterFrame(dt)
	if self.dataRecv then
        -- printf("------------dataRecv")
		if self.requesting == UpdateScene.FLIST_FILENAME then
            printf("------------dataRecv flist")
			io.writefile(self.newListFile, self.dataRecv)
			self.dataRecv = nil

			self.fileListNew = dofile(self.newListFile)
			if self.fileListNew == nil then
				-- print(self.newListFile ..": Open Error!")
				self:endProcess()
				return
			end

            printf("------------oldVersion: %s", self.fileList.ver)
			printf("------------newVersion: %s", self.fileListNew.ver)
			if self.fileListNew.ver == self.fileList.ver then
				self:endProcess()
				return
			end

            self.numFileCheck = 0
            self.requesting = "files"
            self:reqNextFile()
            return
		end

        if self.requesting == "files" then
            printf("------------dataRecv files: %s", self.curStageFile.name)
            --这里假定除了flist以外的文件都在版本目录之下，统一移除版本目录
            -- local fnWithoutVer = string.sub(self.curStageFile.name, string.find(self.curStageFile.name, "/") + 1, #self.curStageFile.name)
            -- local fn = self.path .. fnWithoutVer .. UpdateScene.UPDATE_TEMP_POSTFIX
            local fn = self.path .. self.curStageFile.name .. UpdateScene.UPDATE_TEMP_POSTFIX
            -- printf("------------dataRecv fn: %s", fn)
            local dir = string.match(fn, "(.+)/[^/]*%.%w+$") 
            -- printf("------------dataRecv dir: %s", dir)
            makeDir(dir)
            io.writefile(fn, self.dataRecv)
            self.dataRecv = nil
            if checkFile(fn, self.curStageFile.code) then
                table.insert(UpdateScene.DOWNLOADED_LIST, fn)
                self:reqNextFile()
            else
                self:endProcess()
            end
            return
        end

        return
	end
end

function UpdateScene:onEnter()
    if not UpdateScene.NEED_UPDATE then
        self:runApp()
        return
    end

    if not checkDirOK(self.path) then
        self:runApp()
        return
    end

    self:addNodeEventListener(cc.NODE_ENTER_FRAME_EVENT, function(dt) self:onEnterFrame(dt) end)
    self:scheduleUpdate()

    self.curListFile = self.path .. UpdateScene.FLIST_FILENAME
    -- printf("------------curListFile: %s", self.curListFile)
	self.fileList = nil
	if io.exists(self.curListFile) then
        -- printf("------------curListFile exist")
		self.fileList = dofile(self.curListFile)
	end
	if self.fileList == nil then
		self.fileList = {
			ver = "1.0.0",
			stage = {},
			remove = {},
		}
	end

	self.requestCount = 0
	self.requesting = UpdateScene.FLIST_FILENAME
    self.newListFile = self.curListFile .. UpdateScene.UPDATE_TEMP_POSTFIX
	self.dataRecv = nil
    -- printf("------------newListFile: %s", self.newListFile)
    -- printf("------------requesting: %s", self.requesting)
	self:requestFromServer(self.requesting)
end

function UpdateScene:onExit()
end

function UpdateScene:endProcess()
	print("------------UpdateScene:endProcess")

    if self.fileList and self.fileList.stage then
        local checkOK = true
        for i,v in ipairs(self.fileList.stage) do
            if not checkFile(self.path..v.name, v.code) then
                print("------------Check Files Error")
                checkOK = false
                break
            end
        end

        if checkOK then
            for i, v in ipairs(self.fileList.stage) do
                if v.act =="load" then
                    cc.LuaLoadChunksFromZIP(self.path..v.name)
                end
            end
            for i,v in ipairs(self.fileList.remove) do
                removeFile(self.path..v)
            end
        else
            removeFile(self.curListFile)
        end
    end

    self:runApp()
end

function UpdateScene:requestFromServer(filename, waittime)
    local url = UpdateScene.SERVER_URL .. filename .. UpdateScene.PARAM
    printf("------------URL: %s", url)
    self.requestCount = self.requestCount + 1
    local index = self.requestCount
    local request = network.createHTTPRequest(function(event)
            self:onResponse(event, index, false)
        end, url, "GET")
    request:setTimeout(waittime or 30)
    request:start()
end

function UpdateScene:onResponse(event, index, dumpResponse)
    local request = event.request
    printf("------------REQUEST %d - event.name = %s", index, event.name)
    if event.name == "completed" then
        printf("------------REQUEST %d - getResponseStatusCode() = %d", index, request:getResponseStatusCode())
        --printf("REQUEST %d - getResponseHeadersString() =\n%s", index, request:getResponseHeadersString())

        if request:getResponseStatusCode() ~= 200 then
        	self:endProcess()
        else
            printf("------------REQUEST %d - getResponseDataLength() = %d", index, request:getResponseDataLength())
            if dumpResponse then
                printf("------------REQUEST %d - getResponseString() =\n%s", index, request:getResponseString())
            end
            self.dataRecv = request:getResponseData()
        end
    elseif event.name ~= "progress" then
        printf("------------REQUEST %d - getErrorCode() = %d, getErrorMessage() = %s", index, request:getErrorCode(), request:getErrorMessage())
        self:endProcess()
    end

    print("------------")
end

function UpdateScene:runApp()
    print("------------UpdateScene:runApp")
    require("AppMain")
end


if not UpdateScene.NEED_UPDATE then
    require("AppMain")
else
    display.replaceScene(UpdateScene.new())
end
